#include <windows.h>		// Header File For Windows

/////////////////////////////////////////////////////////////////////////////
// Include the OpenGL headers
#include "gl\gl.h"

#include "stereoscopic_projection.h"

void stereo_projection(GLfloat xmin,GLfloat xmax,GLfloat ymin,
GLfloat ymax,GLfloat znear,GLfloat zfar,GLfloat zzps,GLfloat dist,
GLfloat eye)
{
/* This routine performs the perspective projection for one eye's subfield.
   The projection is in the direction of the negative z axis.

   xmin, ymax, ymin, ymax = the coordinate range, in the plane of zero
   parallax setting, that will be displayed on the screen. The ratio between
   (xmax-xmin) and (ymax-ymin) should equal the aspect ration of the display.

   znear, zfar = the z-coordinate values of the clipping planes.

   zzps = the z-coordinate of the plane of zero parallax setting.

   dist = the distance from the center of projection to the plane of zero
   parallax.

   eye = half the eye separation; positive for the right eye subfield,
   negative for the left eye subfield. */
GLfloat xmid , ymid, clip_near, clip_far;
GLfloat  top, bottom, left ,right;
GLfloat dx , dy , n_over_d;

  dx = xmax - xmin;
  dy = ymax - ymin;

  xmid = (xmax + xmin) / 2;
  ymid = (ymax + ymin) / 2;

  clip_near = dist + zzps - znear;
  clip_far = dist + zzps - zfar;

  n_over_d = clip_near / dist;

  top = n_over_d * dy / 2;
  bottom = -top;
  right = n_over_d * ((dx / 2) - eye);
  left = n_over_d * ((-dx / 2) - eye);
  
  glMatrixMode(GL_PROJECTION);
   
  glLoadIdentity();
  
  glFrustum( left, right, bottom, top, clip_near, clip_far);

  glTranslatef( -xmid - eye, -ymid, -zzps - dist);
  
}

void stereo_projection_picking(GLfloat xmin,GLfloat xmax,GLfloat ymin,
GLfloat ymax,GLfloat znear,GLfloat zfar,GLfloat zzps,GLfloat dist,
GLfloat eye)
{
/* This routine performs the perspective projection for one eye's subfield.
   The projection is in the direction of the negative z axis.

   xmin, ymax, ymin, ymax = the coordinate range, in the plane of zero
   parallax setting, that will be displayed on the screen. The ratio between
   (xmax-xmin) and (ymax-ymin) should equal the aspect ration of the display.

   znear, zfar = the z-coordinate values of the clipping planes.

   zzps = the z-coordinate of the plane of zero parallax setting.

   dist = the distance from the center of projection to the plane of zero
   parallax.

   eye = half the eye separation; positive for the right eye subfield,
   negative for the left eye subfield. */
GLfloat xmid , ymid, clip_near, clip_far;
GLfloat  top, bottom, left ,right;
GLfloat dx , dy , n_over_d;

  dx = xmax - xmin;
  dy = ymax - ymin;

  xmid = (xmax + xmin) / 2;
  ymid = (ymax + ymin) / 2;

  clip_near = dist + zzps - znear;
  clip_far = dist + zzps - zfar;

  n_over_d = clip_near / dist;

  top = n_over_d * dy / 2;
  bottom = -top;
  right = n_over_d * (dx / 2 - eye);
  left = n_over_d * (-dx / 2 - eye);

//we can not delete the picking matrix !!!!!!
/*
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity();
*/
  
  glFrustum (left, right, bottom, top, clip_near, clip_far);

  glTranslatef (-xmid - eye, -ymid, -zzps - dist);
  
}

