
// Copyright  2000, Michal Husak, husakm@vscht.cz
//========================================================================================

// standard Windows and Common Contrlol headers
#include <windows.h>
#include <commdlg.h>
#include <commctrl.h>
#include <string.h>
#include <stdio.h>

#include "dialog.h"
#include "resource.h"
#include "MCE_VR.h"

// state variable
MceVrConfiguration LocalData;
static BOOL			bConfigurationChanged = FALSE;

//////////////////////////////////////////////////////////////////////////////
static BOOL SetSourceFile (HWND hWnd)
{
// this routine simply gets an filename of something
	BOOL								bSuccess = FALSE;
	static OPENFILENAME			Ofn;
	static char									szPath[_MAX_PATH] = "";

	// prepare the open file structure to set the input filename
	memset(&Ofn, 0, sizeof(OPENFILENAME));
	Ofn.lStructSize = sizeof(OPENFILENAME); 
	Ofn.hwndOwner = NULL; 
	Ofn.hInstance = NULL; 
	Ofn.lpstrFilter = "Fou ASCI ELD map files (*.fou)\0*.fou\0All files (*.*)\0*.*\0";
	Ofn.lpstrCustomFilter = NULL; 
	Ofn.lpstrFileTitle = NULL; 
	Ofn.nMaxFileTitle = 0; 
	Ofn.nMaxCustFilter = 0; 
	Ofn.nFilterIndex = 0; 
	Ofn.lpstrInitialDir = NULL; 
	Ofn.lpstrTitle = NULL; 
	Ofn.Flags = OFN_FILEMUSTEXIST | OFN_EXPLORER | OFN_HIDEREADONLY;
	Ofn.nFileOffset = 0;
	Ofn.nFileExtension = 0;
	Ofn.lpstrDefExt = NULL;
	Ofn.lCustData = 0;
	Ofn.lpfnHook = NULL;
	Ofn.lpTemplateName = NULL;


	Ofn.lpstrInitialDir =szPath;
	sprintf(LocalData.szFileName,"%s",""); // null the filename
	Ofn.lpstrFile = LocalData.szFileName;	
	Ofn.nMaxFile = _MAX_PATH;
	
	if (GetOpenFileName(&Ofn))
	{	
	bSuccess = TRUE;
	}
	else
	{
	MessageBox(hWnd, "I could not open this file", "Error", MB_ICONERROR);
	}
	return bSuccess;
}


//////////////////////////////////////////////////////////////////////
// DoSetupDialog
BOOL FAR PASCAL DoSetupDialog( HWND hwnd, UINT message, UINT wParam, LONG lParam )
{
	WORD						wNotifyCode;
	BOOL            bPanelInitialized=FALSE;

    switch( message )
    {
        case WM_INITDIALOG:
           return (LRESULT) 1;

		case WM_MOVE:
			// nothing to do
		break;
		
		case WM_COMMAND:
			wNotifyCode = HIWORD(wParam);
			switch (LOWORD(wParam))
			{
				case ID_OK:
					bConfigurationChanged = TRUE; // OK indicates that we will apply the changes

					EndDialog( hwnd, 0);
					return (LRESULT) 1;
				break;

				case ID_CANCEL:			
					bConfigurationChanged = FALSE; // False indicates that the changes will not be applied 
					
					EndDialog( hwnd, 0);
					return (LRESULT) 1;
				break;

				case ID_INPUT_FILE:
					SetSourceFile(hwnd);
				break;
				
				// selection of output garpahic resolution
				case RES_WINDOWED:
				case RES_640_480:																					
				case RES_800_600:
				case RES_1024_768:
				case RES_1152_864:
					LocalData.OutputResolutionCode=LOWORD(wParam);
				break;
				
				// selection of output garpahic resolution
				case RES_16_BPP:																					
				case RES_24_BPP:
				case RES_32_BPP:
					LocalData.BppCode=LOWORD(wParam);
				break;

				// selection of stereoscopic mode
				case IDC_MODE_MONO:
				case IDC_MODE_STEREO_HW:
				case IDC_MODE_STEREO_INTERLACED:
				case IDC_MODE_STEREO_VERTICAL:
					LocalData.VisualisationMode=LOWORD(wParam);
				break;

				case IDC_CONTROL_KEYBOARD:
				case IDC_CONTROL_JOYSTICK:
				case IDC_CONTROL_VFX3D_KEYBOARD:
				case IDC_CONTROL_VFX3D_CYBERPUCK:
				case IDC_CONTROL_VFX3D_JOYPAD:
					LocalData.ControlMode=LOWORD(wParam);
				break;

				case IDC_JOY1:
				case IDC_JOY2:
					LocalData.JoysticDeviceNumber=LOWORD(wParam);
				break;

				case IDC_OSD_YES:
				case IDC_OSD_NO:
					LocalData.OSD_mode=LOWORD(wParam);
				break;

				case IDC_LINE1:
				case IDC_LINE2:
				case IDC_LINE3:
					LocalData.LineWidthCode=LOWORD(wParam);
				break;

				case IDC_M_SQUARE_XYZ:
				case IDC_M_SQUARE_Z:
				case IDC_M_SQUARE_XZ:
				case IDC_M_CUBE_TRIANGLE:
				case IDC_M_CUBE_WEB:
					LocalData.GridSelection=LOWORD(wParam);
				break;
									
				default:  // another exit, usualy by X button pressing
						
					bConfigurationChanged = FALSE; // False indicates that the changes will not be applied 
					
					EndDialog( hwnd, 0);
					return (LRESULT) 1;											
				break;				

				}
		break;
			
		case WM_PAINT:
				if (bPanelInitialized==FALSE)
				{	
					SendMessage(GetDlgItem(hwnd, LocalData.OutputResolutionCode), BM_SETCHECK , BST_CHECKED , 0);
					SendMessage(GetDlgItem(hwnd, LocalData.BppCode), BM_SETCHECK , BST_CHECKED , 0);		
					SendMessage(GetDlgItem(hwnd, LocalData.VisualisationMode), BM_SETCHECK , BST_CHECKED , 0);		
					SendMessage(GetDlgItem(hwnd, LocalData.ControlMode), BM_SETCHECK , BST_CHECKED , 0);
					SendMessage(GetDlgItem(hwnd, LocalData.JoysticDeviceNumber), BM_SETCHECK , BST_CHECKED , 0);
					SendMessage(GetDlgItem(hwnd, LocalData.LineWidthCode), BM_SETCHECK , BST_CHECKED , 0);
					SendMessage(GetDlgItem(hwnd, LocalData.OSD_mode), BM_SETCHECK , BST_CHECKED , 0);
					SendMessage(GetDlgItem(hwnd, LocalData.GridSelection), BM_SETCHECK , BST_CHECKED , 0);				
					// avoiding future not nessesery inicialization
					bPanelInitialized=TRUE;
				}
		break;
    }
    return (LRESULT) 0;
} 

//////////////////////////////////////////////////////////////////////
// SetupDialog
void SetupDialog( HINSTANCE hInstance, HANDLE hwnd,MceVrConfiguration *Data)
{
	// copy to local variables
	LocalData.OutputResolutionCode=Data->OutputResolutionCode;
	LocalData.BppCode=Data->BppCode;
	LocalData.VisualisationMode=Data->VisualisationMode;
	LocalData.ControlMode=Data->ControlMode;
	LocalData.JoysticDeviceNumber=Data->JoysticDeviceNumber;
	LocalData.LineWidthCode=Data->LineWidthCode;
	LocalData.OSD_mode=Data->OSD_mode;
	LocalData.GridSelection=Data->GridSelection;
	sprintf(LocalData.szFileName,"%s",Data->szFileName);
    DialogBox( hInstance, MAKEINTRESOURCE( IDD_SETUP_DIALOG ), hwnd, DoSetupDialog);

	// we will apply the changes only when the dialogue was exited by ID_OK
	if (bConfigurationChanged)
	{	
		Data->OutputResolutionCode=LocalData.OutputResolutionCode;
		Data->BppCode=LocalData.BppCode;
		Data->VisualisationMode=LocalData.VisualisationMode;
		Data->ControlMode=LocalData.ControlMode;
		Data->JoysticDeviceNumber=LocalData.JoysticDeviceNumber;
		Data->LineWidthCode=LocalData.LineWidthCode;
		Data->OSD_mode=LocalData.OSD_mode;
		Data->GridSelection=LocalData.GridSelection;
		sprintf(Data->szFileName,"%s",LocalData.szFileName);
	}
} 
