// not including cause compilation trouble ...
#include <msquare_mh.h>

/*
   Given a grid cell and an isolevel, calculate the triangular
   facets required to represent the isosurface through the cell.
   Return the number of triangular facets, the array "triangles"
   will be loaded up with the vertices at most 5 triangular facets.
        0 will be returned if the grid cell is either totally above
   of totally below the isolevel.
*/

#define ABS(x) (x < 0 ? -(x) : (x))

XY VertexInterp2D(float isolevel,XY p1,XY p2,float valp1,float valp2)
{
   float mu;
   XY p;

   if (ABS(isolevel-valp1) < 0.00001)
      return(p1);
   if (ABS(isolevel-valp2) < 0.00001)
      return(p2);
   if (ABS(valp1-valp2) < 0.00001)
      return(p1);

   mu = (isolevel - valp1) / (valp2 - valp1);
   p.x =float( p1.x + mu * (p2.x - p1.x));
   p.y =float( p1.y + mu * (p2.y - p1.y));

   return(p);
}

#define ABS(x) (x < 0 ? -(x) : (x))

int Polygonise2D(GRIDCELL2D grid,float isolevel,LINE2D *linefield)
{
   int nlines;
   int squareindex;
   int square_top[2][2];
   int i,j;

   /*edge	numbering scheme
'                 2
'             3------2
'             |      |
'            3|      |1
'             |      |
'             0------1
'                 0
	*/

/*
   XYZ vertlist[12];



   /*
      Determine the index into the edge table which
      tells us which vertices are inside of the square
   */

   squareindex = 0;
   if (grid.val[0] < isolevel) squareindex |= 1;
   if (grid.val[1] < isolevel) squareindex |= 2;
   if (grid.val[2] < isolevel) squareindex |= 4;
   if (grid.val[3] < isolevel) squareindex |= 8;
   
   /* Square is entirely in/out of the surface */

   if ((squareindex == 0) | (squareindex== 15))
      return(0);

   switch (squareindex)
   {
	case 1:
		nlines= 1;
		square_top[0][0] = 0;
		square_top[0][1] = 3;
	break;

	case  2:
		nlines = 1;
		square_top[0][0] = 0;
		square_top[0][1] = 1;
	break;
	
	case 3:
		nlines = 1;
		square_top[0][0] = 3;
		square_top[0][1] = 1;
	break;
	
	case 4:
		nlines = 1;
		square_top[0][0] = 1;
		square_top[0][1] = 2;
	break;
  
	case 5:
		nlines = 2;
		square_top[0][0] = 0;
		square_top[0][1] = 3;
		square_top[1][0] = 1;
		square_top[1][1] = 2;
	break;

	case 6:
		nlines = 1;
		square_top[0][0] = 0;
		square_top[0][1] = 2;
	break;
	
	case  7:
		nlines = 1;
		square_top[0][0] = 3;
		square_top[0][1] = 2;
	break;
  
	case 8:
		nlines = 1;
		square_top[0][0] = 3;
		square_top[0][1] = 2;
	break;

	case 9:
		nlines = 1;
		square_top[0][0] = 0;
		square_top[0][1] = 2;
	break;

	case 10:
		nlines = 2;
		square_top[0][0] = 0;
		square_top[0][1] = 1;
		square_top[1][0] = 2;
		square_top[1][1] = 3;
	break;

	case 11:
		nlines = 1;
		square_top[0][0] = 1;
		square_top[0][1] = 2;
	break;


	case 12:
		nlines = 1;
		square_top[0][0]	= 1;
		square_top[0][1] = 3;
	break;

	case 13:
		nlines = 1;
		square_top[0][0]	= 0;
		square_top[0][1] = 1;
	break;

	case 14:
		nlines = 1;
		square_top[0][0] = 0;
		square_top[0][1] = 3;
	break;
   }

   for (i=0;i<=nlines;i++)
   {
	 for(j=0;j<=1;j++) // first and second line point
	 {
	   switch (square_top[i][j])
	   {
		   case 0:
				linefield[i].p[j]=VertexInterp2D(isolevel,grid.p[0],grid.p[1],grid.val[0],grid.val[1]);
		   break;

		   case 1:
   				linefield[i].p[j]=VertexInterp2D(isolevel,grid.p[1],grid.p[2],grid.val[1],grid.val[2]);
		   break;

		   case 2:
			   	linefield[i].p[j]=VertexInterp2D(isolevel,grid.p[2],grid.p[3],grid.val[2],grid.val[3]);
		   break;

		   case 3:
			   	linefield[i].p[j]=VertexInterp2D(isolevel,grid.p[3],grid.p[0],grid.val[3],grid.val[0]);
		   break;
	   }
	 }			
   }
	return nlines;
}

