//#include <windows.h> // BOOL definition
//#include "MCE_VR.h"
#include <voxel_procesing.h>

// for debuging
#include "log.h"
#include <stdio.h>

// matematic
#include <float.h> // float number limits

XYZ IndToCoord3Df(float x,float y,float z,ELD_MapData *MapData)
{
XYZ result;

	result.x=(MapData->AxisParam[0].Start + x * MapData->AxisParam[0].Step);

	result.y=(MapData->AxisParam[1].Start + y * MapData->AxisParam[1].Step);

	result.z=(MapData->AxisParam[2].Start + z * MapData->AxisParam[2].Step);

return result;
}

XYZ IndToCoord3Di(int x,int y,int z,ELD_MapData *MapData)
{
XYZ result;

	result.x=(MapData->AxisParam[0].Start + x * MapData->AxisParam[0].Step);

	result.y=(MapData->AxisParam[1].Start + y * MapData->AxisParam[1].Step);

	result.z=(MapData->AxisParam[2].Start + z * MapData->AxisParam[2].Step);

return result;
}

XYZ Coord3DfToInd(float x,float y,float z,ELD_MapData *MapData)
{
XYZ result;

	result.x=(x-MapData->AxisParam[0].Start)/MapData->AxisParam[0].Step;

	result.y=(y-MapData->AxisParam[1].Start)/MapData->AxisParam[1].Step;

	result.z=(z-MapData->AxisParam[2].Start)/MapData->AxisParam[2].Step;

return result;
}


void GenerateTrianglesFromVoxels(unsigned int *iNtriangles,
								 ELD_MapData *MapData,
								 float layer_limits[3],
								 TRIANGLE triangles[3][MAXTRIANGLES])
{
	int x;
	int y;
	int z;
	int xmax;
	int ymax;
	int zmax;
	int i;

	int iNewTriangles;
	GRIDCELL grid;

#ifdef _DEBUG
	LogMsg("GenerateTrianglesFromVoxels\n");
#endif
	char szBuffer[256];

	xmax=MapData->Lines-1;
	ymax=MapData->Columns-1;
	zmax=MapData->Sections-1;
		
	for (x=0;x<xmax;x++)
	{
		for (y=0;y<ymax;y++)
		{
			for (z=0;z<zmax;z++)
			{
				grid.p[0]=IndToCoord3Di(x,y,z,MapData);
				grid.val[0]=MapData->Voxel[x][y][z];

				grid.p[1]=IndToCoord3Di(x+1,y,z,MapData);
				grid.val[1]=MapData->Voxel[x+1][y][z];

				grid.p[2]=IndToCoord3Di(x+1,y+1,z,MapData);
				grid.val[2]=MapData->Voxel[x+1][y+1][z];

				grid.p[3]=IndToCoord3Di(x,y+1,z,MapData);
				grid.val[3]=MapData->Voxel[x][y+1][z];

				grid.p[4]=IndToCoord3Di(x,y,z+1,MapData);
				grid.val[4]=MapData->Voxel[x][y][z+1];

				grid.p[5]=IndToCoord3Di(x+1,y,z+1,MapData);
				grid.val[5]=MapData->Voxel[x+1][y][z+1];

				grid.p[6]=IndToCoord3Di(x+1,y+1,z+1,MapData);
				grid.val[6]=MapData->Voxel[x+1][y+1][z+1];

				grid.p[7]=IndToCoord3Di(x,y+1,z+1,MapData);
				grid.val[7]=MapData->Voxel[x][y+1][z+1];

				// we will do it for all levels now
				// to speed up the execution 
				// by avoiding repeating cell values operations

				for (i=0;i<=2;i++)
				{
					if ((iNtriangles[i]+10)<MAXTRIANGLES)
					{
						// WARNING - very pointer sensitive operation: &triangles 
						iNewTriangles= Polygonise(grid,layer_limits[i],&triangles[i][iNtriangles[i]]);
						iNtriangles[i]=iNtriangles[i]+iNewTriangles;
					}
					else
					{
						sprintf(szBuffer,"iNtriangles[i]+2 <MAXCONTOURLINES = %d \n",iNtriangles[i]+2);
						LogMsg(szBuffer);	
						return;
					}
#ifdef _DEBUG
					sprintf(szBuffer,"Layer: %d iNewTriangles %d  iNtriangles[i] %d\n",
					i,iNewTriangles,iNtriangles[i]);
					LogMsg(szBuffer);
#endif
				}
				
			}
		}	
	}
}

void SetLayerLimits(float zero,float step,float *layer_limits,ELD_MapData *MapData)
{
	int x;
	int y;
	int z;
	int xmax;
	int ymax;
	int zmax;
	float maximum=FLT_MIN;
	float minimum=FLT_MAX;
	float value;

	xmax=MapData->Lines-1;
	ymax=MapData->Columns-1;
	zmax=MapData->Sections-1;

	for (x=0;x<xmax;x++)
	{
		for (y=0;y<ymax;y++)
		{
			for (z=0;z<zmax;z++)
			{
				value=MapData->Voxel[x][y][z];
				if (value >= maximum) maximum = value;
				if (value <= minimum) minimum =	value;
			}
		}
	}

	layer_limits[0]=(float)(minimum+zero*(maximum-minimum));
	layer_limits[1]=(float)(minimum+(zero+step)*(maximum-minimum));
	layer_limits[2]=(float)(minimum+(zero+2*step)*(maximum-minimum));

	MapData->maximum=maximum;
	MapData->minimum=minimum;
}


void VoxelDirectionalContour(unsigned int *iNcontourlines,
							 ELD_MapData *MapData,
							 float layer_limits[3],
							 LINE	contourlines[3][MAXCONTOURLINES],
							 contour_direction direction)
{
// Warning - trick used:
// to speed up the calulation the line generation
// is calculated in coordinate system derivated from the
// voxel field indexes
// and then transformed to real spece value
	int x;
	int y;
	int z;
	int xmax;
	int ymax;
	int zmax;
	int i,j,k;

	int iNewLines;
	GRIDCELL2D grid;
	LINE2D newlines[2];

#ifdef _DEBUG
	LogMsg("CalulateDirectionalContour()\n");
#endif

	char szBuffer[256];

	xmax=MapData->Lines;
	ymax=MapData->Columns;
	zmax=MapData->Sections;
	
	switch(direction)
	{
	case X:// map in X direction, x,y of the 2D grid coresponds to 3D z,y
		{ 
		 for (x=0;x<xmax;x++)
		 {
	      for (y=0;y<(ymax-1);y++)
		  {
	       for (z=0;z<(zmax-1);z++)
		   { // prepare the grid values		
		   grid.p[0].x=(float)z;
		   grid.p[0].y=(float)y;
		   grid.val[0]=MapData->Voxel[x][y][z];
	
		   grid.p[1].x=(float)z+1;
		   grid.p[1].y=(float)y;
		   grid.val[1]=MapData->Voxel[x][y][z+1];

		   grid.p[2].x=(float)z+1;
		   grid.p[2].y=(float)y+1;
		   grid.val[2]=MapData->Voxel[x][y+1][z+1];

		   grid.p[3].x=(float)z;
		   grid.p[3].y=(float)y+1;
		   grid.val[3]=MapData->Voxel[x][y+1][z];
				
		   for (i=0;i<=2;i++)
		   {
		    if ((iNcontourlines[i]+2)<MAXCONTOURLINES) // check the max avaaliable number of lines
			{
			 iNewLines= Polygonise2D(grid,layer_limits[i],&newlines[0]);
			 // copy lines from the buffer to destination
			 for (j=0;j<iNewLines;j++)
			 {
			  for(k=0;k<=1;k++) // copy the begining and end of the line to destination
			  {
				contourlines[i][iNcontourlines[i]+j].p[k]=
				IndToCoord3Df((float)x,newlines[j].p[k].y,newlines[j].p[k].x,MapData);
			  }// next k
			 }// next j
			 iNcontourlines[i]=iNcontourlines[i]+iNewLines;
			}// endif
			else
			{
				sprintf(szBuffer,"iNcontourlines[i]+2 <MAXCONTOURLINES = %d \n",iNcontourlines[i]+2);
				LogMsg(szBuffer);	
				return;
			}
		   }// next i							
		  }// next y
		 }// next x
		}// next z
		}// end case
#ifdef _DEBUG
	LogMsg("Finished X directional contour\n");
#endif
	break;

	case Y:// map in Y direction, x,y of the 2D grid coresponds to 3D x,z
		{
		 for (y=0;y<ymax;y++)
		 {
	      for (x=0;x<(xmax-1);x++)
		  {
	       for (z=0;z<(zmax-1);z++)
		   { // prepare the grid values		
		   grid.p[0].x=(float)x;
		   grid.p[0].y=(float)z;
		   grid.val[0]=MapData->Voxel[x][y][z];

		   grid.p[1].x=(float)x+1;
		   grid.p[1].y=(float)z;
		   grid.val[1]=MapData->Voxel[x+1][y][z];

		   grid.p[2].x=(float)x+1;
		   grid.p[2].y=(float)z+1;;
		   grid.val[2]=MapData->Voxel[x+1][y][z+1];

		   grid.p[3].x=(float)x;
		   grid.p[3].y=(float)z+1;
		   grid.val[3]=MapData->Voxel[x][y][z+1];
				
		   for (i=0;i<=2;i++)
		   {
		    if ((iNcontourlines[i]+2)<MAXCONTOURLINES) // check the max avaaliable number of lines
			{
			 iNewLines= Polygonise2D(grid,layer_limits[i],&newlines[0]);
			 // copy lines from the buffer to destination
			 for (j=0;j<iNewLines;j++)
			 {
			  for(k=0;k<=1;k++) // copy the begining and end of the line to destination
			  {
				contourlines[i][iNcontourlines[i]+j].p[k]=
				IndToCoord3Df(newlines[j].p[k].x,(float)y,newlines[j].p[k].y,MapData);
			  }// next k
			 }// next j
			 iNcontourlines[i]=iNcontourlines[i]+iNewLines;
			}// endif
			else
			{
				sprintf(szBuffer,"iNcontourlines[i]+2 <MAXCONTOURLINES = %d \n",iNcontourlines[i]+2);
				LogMsg(szBuffer);
				return;
			}
		   }// next i							
		  }// next y
		 }// next x
		}// next z
		}// end case
#ifdef _DEBUG
	LogMsg("Finished Y directional contour\n");
#endif
	break;

	case Z: // map in Z direction, x,y of the 2D grid coresponds to 3D x,y
		{ 
		 for (z=0;z<zmax;z++)
		 {
	      for (x=0;x<(xmax-1);x++)
		  {
	       for (y=0;y<(ymax-1);y++)
		   { // prepare the grid values		
		   grid.p[0].x=(float)x;
		   grid.p[0].y=(float)y;
		   grid.val[0]=MapData->Voxel[x][y][z];

		   grid.p[1].x=(float)x+1;
		   grid.p[1].y=(float)y;
		   grid.val[1]=MapData->Voxel[x+1][y][z];

		   grid.p[2].x=(float)x+1;
		   grid.p[2].y=(float)y+1;
		   grid.val[2]=MapData->Voxel[x+1][y+1][z];

		   grid.p[3].x=(float)x;
		   grid.p[3].y=(float)y+1;
		   grid.val[3]=MapData->Voxel[x][y+1][z];
				
		   for (i=0;i<=2;i++)
		   {
		    if ((iNcontourlines[i]+2)<MAXCONTOURLINES) // check the max avaaliable number of lines
			{
			 iNewLines= Polygonise2D(grid,layer_limits[i],&newlines[0]);
			 // copy lines from the buffer to destination
			 for (j=0;j<iNewLines;j++)
			 {
			  for(k=0;k<=1;k++) // copy the begining and end of the line to destination
			  {
				contourlines[i][iNcontourlines[i]+j].p[k]=
				IndToCoord3Df(newlines[j].p[k].x,newlines[j].p[k].y,(float)z,MapData);
			  }// next k
			 }// next j
			 iNcontourlines[i]=iNcontourlines[i]+iNewLines;
			}// endif
			else
			{
				sprintf(szBuffer,"iNcontourlines[i]+2 <MAXCONTOURLINES = %d \n",iNcontourlines[i]+2);
				LogMsg(szBuffer);	
				return;
			}
		   }// next i							
		  }// next y
		 }// next x
		}// next z
		}// end case
#ifdef _DEBUG
	LogMsg("Finished Z directional contour\n");
#endif
	break;
	}
#ifdef _DEBUG
	LogMsg("Directional contour finished\n");
#endif
}
